from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

class Experiments(QWidget):
    """
    Experiment box widget
    """

    def __init__(self, name, parent):

        super().__init__()

        self._name = name
        self._fitter = parent._fitter
        self._slider_list = parent._slider_list
        self._global_var = parent._global_var
        self._global_tracker = parent._global_tracker
        self._connectors_seen = parent._connectors_seen
        self._global_connectors = parent._global_connectors
        self._plot_frame = parent._plot_frame

        self.layout()

    @property
    def name(self):
        """
        """
        return self._name

    def layout(self):
        """
        """
        self._main_layout = QVBoxLayout(self)
        self._header_layout = QHBoxLayout()

        # Construct the header for the experiment
        self._name_label = QLabel(self._name)

        # Button to hide and show advanced options for the experiment
        self._show_options_button = QPushButton("Show Sliders", self)
        self._show_options_button.clicked.connect(self.slider_popup)

        # Button to remove experiment
        self._remove_name = "Remove"
        self.exp_widgets()
        
        self._remove_button = QPushButton(self._remove_name, self)
        self._remove_button.clicked.connect(self.remove)

        # add exp name, remove and show sliders buttons to layout
        self._header_layout.addWidget(self._name_label)
        self._header_layout.addStretch(1)
        self._header_layout.addWidget(self._show_options_button)
        self._header_layout.addWidget(self._remove_button)
        self._header_layout.addStretch(2)

        self._main_layout.addLayout(self._header_layout)

        # Create empty box for any required parameters
        self._req_box = QFrame()
        self._req_layout = QVBoxLayout()
        self._req_box.setLayout(self._req_layout)
        self._main_layout.addWidget(self._req_box)

        # Create divider    
        self._divider = QFrame()
        self._divider.setFrameShape(QFrame.HLine)
        self._main_layout.addWidget(self._divider)

        self.setFixedHeight(110)

    def exp_widgets(self):
        """
        for changing local/global specific items
        """
        pass

    def remove(self):
        """
        """
        pass

    def slider_popup(self):
        """
        show window containing sliders
        """
        pass

            


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

from .. import sliders
from .base import SliderPopUp

class GlobalPopUp(SliderPopUp):
    """
    pop-up window for slider widgets
    """

    def __init__(self, parent):
        """
        """
        super().__init__(parent)

    def populate(self):
        """
        """
        sliders = self._slider_list["Global"][self._name]

        # add sliders to layout
        self._main_layout.addWidget(sliders)


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

from .. import sliders

class SliderPopUp(QDialog):
    """
    pop-up window for slider widgets
    """

    def __init__(self, parent):
        """
        """
        super().__init__()

        self._name = parent._name
        self._exp = parent._exp
        self._fitter = parent._fitter
        self._slider_list = parent._slider_list
        
        self.layout()

    def layout(self):
        """
        """
        self._main_layout = QVBoxLayout(self)
        self.setWindowTitle(self._name)

        self.populate()

    def populate(self):
        """
        """
        pass


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect, math

class Sliders(QWidget):
    """
    create sliders for an experiment
    """

    def __init__(self, param_name, parent):
        super().__init__()

        self._exp = parent._exp
        self._param_name = param_name
        self._fitter = parent._fitter
        self._fit_run = False
        self._main_box = parent._main_box
        self._plot_frame = parent._plot_frame

        self.layout()

    @property
    def name(self):
        """
        """
        return self._param_name

    def layout(self):
        """
        """
        self._main_layout = QGridLayout(self)
        self._main_layout.setVerticalSpacing(40)

        self._name_label = QLabel(self._param_name, self)
        self._main_layout.addWidget(self._name_label, 0, 0, 0, 2)

        self._fix = QCheckBox("Fix?", self)
        self._fix.toggle()
        self._fix.setChecked(False)
        self._fix.stateChanged.connect(self.fix_layout)
        self._main_layout.addWidget(self._fix, 1, 0)

        self._slider = QSlider(Qt.Horizontal)
        self._slider.sliderReleased.connect(self.update_val)
        self._main_layout.addWidget(self._slider, 1, 1)
        self._slider.setMinimumWidth(100)

        self._param_guess_label = QLabel("", self)
        self._main_layout.addWidget(self._param_guess_label, 1, 2)

        self.bounds()

        self._fix_int = QLineEdit(self)
        self._main_layout.addWidget(self._fix_int, 1, 3)
        self._fix_int.setText(str(1))
        self._fix_int.returnPressed.connect(self.fix)
        self._fix_int.hide()

        self._update_min_label = QLabel("min: ", self)
        self._main_layout.addWidget(self._update_min_label, 1, 4)

        self._update_min = QLineEdit(self)
        self._main_layout.addWidget(self._update_min, 1, 5)
        self._update_min.returnPressed.connect(self.min_bounds)
        self._update_min.setFixedWidth(60)

        self._update_max_label = QLabel("max: ", self)
        self._main_layout.addWidget(self._update_max_label, 1, 6)

        self._update_max = QLineEdit(self)
        self._main_layout.addWidget(self._update_max, 1, 7)
        self._update_max.returnPressed.connect(self.max_bounds)
        self._update_max.setFixedWidth(60)

    @pyqtSlot()
    def set_fit_true(self):
        """
        """
        self._fit_run = True

    def check_if_fit(self):
        """
        if a fit has been run, and a slider is changed, change all parameters back to guesses in slider widgets
        """
        if self._fit_run:
            self._fitter.guess_to_value()
            self._fit_run = False

        self._plot_frame.update()

    def fix_layout(self, state):
        """
        initial parameter fix and updating whether slider/fixed int is hidden or shown
        """
        if state == Qt.Checked:
            # change widget views
            self._fix_int.show()
            self._slider.hide()
            self._param_guess_label.hide()
            self._fitter.update_fixed(self._param_name, int(self._fix_int.text()), self._exp)
            self.check_if_fit()
        else:
            #change widget views
            self._fix_int.hide()
            self._slider.show()
            self._param_guess_label.show()

            self._fitter.update_fixed(self._param_name, None, self._exp)

    def fix(self):
        """
        update fixed value when enter/return key pressed
        """
        try:
            self._fitter.update_fixed(self._param_name, int(self._fix_int.text()), self._exp)
            self.check_if_fit()
        except:
            pass

    def update_val(self):
        """
        update value for parameter based on slider value
        """

        value = int(self._slider.value())

        # transform values back
        if self._range_diff < 10:
            value /= 10
        elif self._range_diff < 100000:
            value *= 100
        elif self._range_diff < 100000000:
            value = 10 ** value

        if value != 0:
            # if guess update, update parameter as well for plot
            self._fitter.update_guess(self._param_name, value, self._exp)
            self._fitter.update_value(self._param_name, value, self._exp)
            self._param_guess_label.setText(str(value))
        else:
            pass

        self.check_if_fit()

    def transform_init(self, val):
        """
        transform values for use in slider
        """
        if self._range_diff < 10:
            new_val = val * 10
        elif self._range_diff < 100000:
            new_val = val / 100
        elif self._range_diff < 100000000:
            new_val = math.log10(val)

        return new_val

    def min_bounds(self):
        """
        update the minimum bounds when enter/return key pressed
        """
        try:
            self._min = int(self._update_min.text())

            # make sure K min bound isn't negative
            if "K" in self._param_name and self._min < 0:
                self._min = 1
                print("K cannot be negative", self._min)

            # set new range
            self._range_diff = self._max - self._min

            # if range has significantly changed, update value transformations
            self._slider_max = self.transform_init(self._max)
            self._slider_min = self.transform_init(self._min)

            # set slider min
            self._slider.setMinimum(self._slider_min)
            self.update_bounds()

            print("min bound updated: " + value)
        except:
            print('invalid value')

    def max_bounds(self):
        """
        update maximum bounds when enter/return key pressed
        """
        try:
            self._max = int(self._update_max.text())

            # set new range
            self._range_diff = self._max - self._min

            # if range has significantly changed, update the value transformations
            self._slider_max = self.transform_init(self._max)
            self._slider_min = self.transform_init(self._min)

            # set slider max
            self._slider.setMaximum(self._slider_max)
            self.update_bounds()

            print("max bound updated: " + value)
        except:
            pass

    def bounds(self):
        """
        for anything specific to child class
        """
        # transform values based on parameter to allow floats to pass to fitter and 
        # make sliders easier to use, QtSlider only allows integers
        self._range_diff = self._max - self._min

        min_range = self.transform_init(self._min)
        max_range = self.transform_init(self._max)

        self._slider.setMinimum(min_range)
        self._slider.setMaximum(max_range)

    def update_bounds(self):
        """
        update min/max bounds and check if range needs to be updated as well
        """
        pass



__author__ = "Hiranmayi Duvvuri"
__date__ = "2017-01-08"
__all__ = ["sliders", "exp_frames", "slider_popup"]

from . import sliders
from . import exp_frames
from . import slider_popup

from .exp_setup import AddExperimentWindow
from .main_window import *
from .fit_update import *
from .sliders import *
from .exp_frames import *
from .slider_popup import *


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

from .base import Experiments
from .. import slider_popup
from .. import sliders

class LocalBox(Experiments):
    """
    hold local parameters/sliders
    """

    def __init__(self, exp, name, parent):

        self._exp = exp
        self._required_fields = {}
        self._experiments = parent._experiments
        self._exp_box = parent._exp_box
        self._main_box = parent

        super().__init__(name, parent)

    def exp_widgets(self):
        """
        create sliders for experiment
        """
        parameters = self._exp.param_values

        for p, v in parameters.items():
            s = sliders.LocalSliders(p, v, self)
            self._slider_list["Local"][self._exp].append(s)

    def slider_popup(self):
        """
        hide and show slider window
        """
        self._slider_window = slider_popup.LocalPopUp(self)
        self._slider_window.show()
                
    def update_req(self):
        """
        checks if any global connectors are connected and updates to add fields for any required data
        """
        exp_connectors = self._connectors_seen[self._exp]

        for c in exp_connectors:
            required = c.required_data
            for i in required:
                if i not in self._required_fields:
                    label_name = str(i).replace("_", " ") + ": "
                    label = QLabel(label_name.title(), self)
                    field = QLineEdit(self)
                    field.setMinimumWidth(150)
                    field.setPlaceholderText("values in kcal/mol or C°")
                    self._required_fields[i] = field

                    stretch = QHBoxLayout()
                    stretch.addWidget(label)
                    stretch.addWidget(field)
                    stretch.addStretch(1)

                    self._req_layout.addLayout(stretch)
                    self.setFixedHeight(self.height()+20)
                else:
                    print("already there")
                    
    def set_attr(self):
        """
        update data from global connector fields
        """
        for n, v in self._required_fields.items():
            try:
                val = float(v.text())
            except:
                val = v.text()

            setattr(self._exp, n, val)

    def remove(self):
        """
        """
        self._fitter.remove_experiment(self._exp)
        self._slider_list["Local"].pop(self._exp, None)
        self.close()



"""
pytc GUI using PyQt5
"""
from pytc.global_fit import GlobalFit

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from .exp_setup import AddExperimentWindow
from .fit_update import AllExp, PlotBox

from matplotlib.backends.backend_pdf import PdfPages

import pickle

class Splitter(QWidget):
    """
    hold main experiment based widgets
    """

    def __init__(self, parent):
        super().__init__()

        self._fitter = parent._fitter

        self.layout()

    def layout(self):
        """
        """
        main_layout = QVBoxLayout(self)

        gen_fit = QPushButton("Fit Experiments", self)
        gen_fit.clicked.connect(self.fit_shortcut)
        button_layout.addWidget(gen_fit)

        self.QLabel()

        main_layout.addLayout(button_layout)

        button_layout = QHBoxLayout(self)
        self._plot_frame = PlotBox(self)
        self._exp_frame = AllExp(self)

        splitter = QSplitter(Qt.Horizontal)
        splitter.addWidget(self._plot_frame)
        splitter.addWidget(self._exp_frame)
        splitter.setSizes([200, 200])

        main_layout.addWidget(splitter)

    def clear(self):
        """
        """
        self._plot_frame.clear()
        self._exp_frame.clear()

    def fit_shortcut(self):
        """
        """
        self._exp_frame.perform_fit()
        self._plot_frame.update()

class Main(QMainWindow):
    """
    """
    def __init__(self):
        super().__init__()

        self._fitter = GlobalFit()

        self.layout()

    def layout(self):
        """
        make the menu bar
        """
        menu = self.menuBar()
        menu.setNativeMenuBar(False)

        file_menu = menu.addMenu("File")
        fitting_commands = menu.addMenu("Fitting")

        fit_exp = QAction("Fit Experiments", self)
        fit_exp.setShortcut("Ctrl+F")
        fit_exp.triggered.connect(self.fit_exp)
        fitting_commands.addAction(fit_exp)

        add_exp = QAction("Add Experiment", self)
        add_exp.setShortcut("Ctrl+Shift+N")
        add_exp.triggered.connect(self.add_file)
        file_menu.addAction(add_exp)

        save_exp = QAction("Export Results", self)
        save_exp.setShortcut("Ctrl+S")
        save_exp.triggered.connect(self.save_file)
        file_menu.addAction(save_exp)

        file_menu.addSeparator()

        new_exp = QAction("New Session", self)
        new_exp.setShortcut("Ctrl+N")
        new_exp.triggered.connect(self.new_exp)
        file_menu.addAction(new_exp)

        close_window = QAction("Close Window", self)
        close_window.setShortcut("Ctrl+W")
        close_window.triggered.connect(self.close_program)
        file_menu.addAction(close_window)

        # add shortcut actions to main window, for qt5 bug
        self.addAction(add_exp)
        self.addAction(fit_exp)
        self.addAction(save_exp)
        self.addAction(new_exp)
        self.addAction(close_window)

        self._exp = Splitter(self)
        self.setCentralWidget(self._exp)

        self.resize(1000, 600)
        self.move(QApplication.desktop().screen().rect().center()-self.rect().center())
        self.setWindowTitle('pytc')
        self.show()

    def print_exp(self):
        """
        testing, check pytc experiments loading
        """
        print(self._fitter.experiments, self._fitter.global_param)

    def fit_exp(self):
        """
        fitting shortcut
        """
        self._exp.fit_shortcut()

    def add_file(self):
        """
        add a new pytc experiment.
        """
        self._new_exp = AddExperimentWindow(self._fitter, self._exp)
        self._new_exp.show()

    def new_exp(self):
        """
        clear everything and start over
        """
        warning_message = QMessageBox.warning(self, "warning!", "Are you sure you want to start a new session?", QMessageBox.Yes | QMessageBox.No)

        if warning_message == QMessageBox.Yes:
            self._exp.clear()
        else:
            print("don't start over!")

    def save_file(self):
        """
        save out fit data and plot
        """

        file_name, _ = QFileDialog.getSaveFileName(self, "Save Experiment Output", "", "Text Files (*.txt);;CSV Files (*.csv)")
        plot_name = file_name.split(".")[0] + "_plot.pdf"

        try:
            data_file = open(file_name, "w")
            data_file.write(self._fitter.fit_as_csv)
            data_file.close()

            plot_save = PdfPages(plot_name)
            fig, ax = self._fitter.plot()
            plot_save.savefig(fig)
            plot_save.close()
        except:
            print("save failed")

    def close_program(self):
        """
        close window
        """
        self.close()



import pytc
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import inspect
import re

class AddExperimentWindow(QDialog):
    """
    add experiment pop-up box
    """

    def __init__(self, fitter, on_close):

        super().__init__()

        subclasses = pytc.indiv_models.ITCModel.__subclasses__()
        self._models = {re.sub(r"(\w)([A-Z])", r"\1 \2", i.__name__): i for i in subclasses}

        self._exp_file = None
        self._shot_start = 1
        self._fitter = fitter

        self._on_close = on_close

        self.layout()

    def layout(self):
        """
        """
        # exp text, model dropdown, shots select
        main_layout = QVBoxLayout(self)
        self._form_layout = QFormLayout()

        self._gen_widgets = {}

        model_select = QComboBox(self)
        model_names = list(self._models.keys())
        model_names.sort()

        for k in model_names:
            model_select.addItem(k)

        self._exp_model = self._models[str(model_select.currentText())]
        model_select.activated[str].connect(self.model_select)

        load_exp = QPushButton("Load File", self)
        load_exp.clicked.connect(self.add_file)

        self._exp_label = QLabel("...", self)

        shot_start_text = QLineEdit(self)
        shot_start_text.setText("0")
        shot_start_text.textChanged[str].connect(self.shot_select)

        gen_exp = QPushButton("OK", self)
        gen_exp.clicked.connect(self.generate)

        self._form_layout.addRow(load_exp, self._exp_label)
        self._form_layout.addRow(QLabel("Select Model:"), model_select)
        self._form_layout.addRow(QLabel("Shot Start:"), shot_start_text)

        self.update_widgets()

        main_layout.addLayout(self._form_layout)
        main_layout.addWidget(gen_exp)

        self.setWindowTitle('Add Experiment to Fitter')

    def update_widgets(self):
        """
        """
        # check for any model specific parameters and update text fields with those values
        for l in self._gen_widgets.values():
            widget_labeled = self._form_layout.labelForField(l)
            if widget_labeled is not None:
                widget_labeled.deleteLater()
            l.deleteLater()

        self.adjustSize()
        self._gen_widgets = {}

        parent_req = pytc.indiv_models.ITCModel()

        sig_parent = inspect.getargspec(parent_req.__init__)
        sig_child = inspect.getargspec(self._exp_model.__init__)

        args = {arg: param for arg, param in zip(sig_child.args[1:], sig_child.defaults)}

        unique = list(set(sig_child.args) - set(sig_parent.args))

        for i in unique:
            self._gen_widgets[i] = QLineEdit(self)
            self._gen_widgets[i].setText(str(args[i]))

        # add widgets to the pop-up box
        for name, entry in self._gen_widgets.items():
            label_name = str(name).replace("_", " ") + ": "
            label = QLabel(label_name.title(), self)

            self._form_layout.addRow(label, entry)

    def model_select(self, model):
        """
        """
        self._exp_model = self._models[model]
        self.update_widgets()

    def shot_select(self, shot):
        """
        """
        try:
            self._shot_start = int(shot)
        except:
            pass

    def add_file(self):
        """
        """
        file_name, _ = QFileDialog.getOpenFileName(self, "Select a file...", "", filter="DH Files (*.DH)")
        self._exp_file = str(file_name)
        self._exp_name = file_name.split("/")[-1]
        self._exp_label.setText(self._exp_name)

    def generate(self):
        """
        """
        if self._exp_file != None:

            # set up dictionary for paramter names and their values in float or int
            model_param = {}
            for k, v in self._gen_widgets.items():
                val = None
                if "." in v.text():
                    val = float(v.text())
                else:
                    val = int(v.text())

                model_param[k] = val

            itc_exp = pytc.ITCExperiment(self._exp_file, self._exp_model, shot_start = self._shot_start, **model_param)
            self._fitter.add_experiment(itc_exp)

            self._on_close._plot_frame.update()
            self._on_close._exp_frame.add_exp()
            self.close()
        else:
            error_message = QMessageBox.warning(self, "warning", "No .DH file provided", QMessageBox.Ok)
            



from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect, math

from .base import Sliders

class GlobalSliders(Sliders):
    """
    create sliders for a global exp object
    """

    def __init__(self, param_name, parent):
        """
        """
        super().__init__(param_name, parent)

    def bounds(self):
        """
        update min/max for slider
        """
        exp_range = self._fitter.param_ranges[0][self._param_name]

        self._min = exp_range[0]
        self._max = exp_range[1]

        super().bounds()

        self._main_box.fit_signal.connect(self.set_fit_true)

    def update_bounds(self):
        """
        update min/max bounds and check if range needs to be updated as well
        """
        bounds = [self._min, self._max]
        self._fitter.update_bounds(self._param_name, bounds, self._exp)

        # check if bounds are smaller than range, then update.
        curr_range = self._exp.model.param_guess_ranges[self._param_name]
        curr_bounds = self._exp.model.bounds[self._param_name]

        if curr_range[0] < curr_bounds[0] or curr_range[1] > curr_bounds[1]:
            self._fitter.update_range(self._param_name, bounds, self._exp)



from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect, math

from .add_global_connector import AddGlobalConnectorWindow
from .base import Sliders
from .. import exp_frames

class LocalSliders(Sliders):
    """
    create sliders for a local exp object
    """

    def __init__(self, param_name, value, parent):

        self._value = value
        self._global_var = parent._global_var
        self._slider_list = parent._slider_list
        self._connectors_seen = parent._connectors_seen
        self._global_connectors = parent._global_connectors
        self._global_tracker = parent._global_tracker
        self._exp_box = parent._exp_box
        self._main_box = parent._main_box
        self._if_connected = None

        super().__init__(param_name, parent)

    def bounds(self):
        """
        update the min max for the slider
        """
        exp_range = self._exp.model.param_guess_ranges[self._param_name]

        self._min = exp_range[0]
        self._max = exp_range[1]

        super().bounds()

        self._link = QComboBox(self)
        self._link.addItem("Unlink")
        self._link.addItem("Add Global Var")
        self._link.addItem("Add Connector")

        # update lists of variables for each new slider, check if var is a string or a connector object
        for i in self._global_var:
            if isinstance(i, str):
                self._link.addItem(i)
            else: 
                for p, v in i.local_methods.items():
                    self._link.addItem(p)

        self._link.activated[str].connect(self.link_unlink)
        self._main_layout.addWidget(self._link, 1, 3)

        self._main_box.fit_signal.connect(self.set_fit_true)

    def link_unlink(self, status):
        """
        add global variable, update if parameter is linked or not to a global paremeter
        """
        if status == "Unlink":
            try:
                self._fitter.unlink_from_global(self._exp, self._param_name)
                self.reset()
                self._global_tracker[self._if_connected].unlinked(self)
            except:
                pass

        elif status == "Add Global Var":
            text, ok = QInputDialog.getText(self, "Add Global Variable", "Var Name: ")
            if ok: 
                self._global_var.append(text)
                for e in self._slider_list["Local"].values():
                    for i in e:
                        i.update_global(text)

                index = self._link.findText(text)
                self._link.setCurrentIndex(index)
                self.global_link(text)
            else:
                i = self._link.findText("Unlink")
                self._link.setCurrentIndex(i)

        elif status == "Add Connector":
    
            def connector_handler(connector,var_names,curr_var):
        
                self._global_var.append(connector)
                for v in var_names:
                    self._global_connectors[v] = [connector.local_methods[v], connector]

                # Append connector methods to dropbdown lists
                for p, v in connector.local_methods.items():
                    for e in self._slider_list["Local"].values():
                        for i in e:
                            i.update_global(p)

                index = self._link.findText(curr_var)
                self._link.setCurrentIndex(index)
                self.connector_link(curr_var)
            
            self.diag = AddGlobalConnectorWindow(connector_handler)
            self.diag.show()

        elif status not in self._global_connectors:
            # connect to a simple global variable
            self.global_link(status)
        else:
            # connect to global connector
            self.connector_link(status)

    def global_link(self,var):
        """
        """
        # connect to a simple global variable
        self._fitter.link_to_global(self._exp, self._param_name, var)
        self._slider.hide()
        self._param_guess_label.hide()
        self._fix.hide()
        self._update_min_label.hide()
        self._update_min.hide()
        self._update_max_label.hide()
        self._update_max.hide()

        # set current connected name
        self._if_connected = var

        # add global exp to experiments widget
        if var not in self._slider_list["Global"]:
            # create global exp object and add to layout
            param_obj = self._fitter.global_param[var]
            global_e = exp_frames.GlobalBox(var, param_obj, self)
            self._global_tracker[var] = global_e
            self._exp_box.addWidget(global_e)

        self._global_tracker[var].linked(self)

    def connector_link(self,var):
        """
        """
        # connect to global connector
        self._slider.hide()
        self._param_guess_label.hide()
        self._fix.hide()
        self._update_min_label.hide()
        self._update_min.hide()
        self._update_max_label.hide()
        self._update_max.hide()

        curr_connector = self._global_connectors[var][1]
        name = curr_connector.name
        self._connectors_seen[self._exp].append(curr_connector)
        self._fitter.link_to_global(self._exp, self._param_name, self._global_connectors[var][0])

        # add connector to experiments widget
        if name not in self._slider_list["Global"]:
            self._slider_list["Global"][name] = []

            # create a connector holder and add to layout
            connector_e = exp_frames.ConnectorsBox(name, curr_connector, self)
            self._exp_box.addWidget(connector_e)
            self._global_tracker[name] = connector_e

        self._global_tracker[name].linked(self)

        # set current connected name
        self._if_connected = name

        # check for instances of LocalBox and update
        for loc_obj in self._exp_box.parentWidget().findChildren(exp_frames.LocalBox):
            loc_obj.update_req()

    def update_global(self, value):
        """
        update the list of global parameters in combobox
        """
        self._link.addItem(value)

    def reset(self):
        """
        if global exp object deleted, return local slider object to unlinked state
        """
        self._slider.show()
        self._param_guess_label.show()
        self._fix.show()
        self._update_min_label.show()
        self._update_min.show()
        self._update_max_label.show()
        self._update_max.show()

        unlink_index = self._link.findText("Unlink", Qt.MatchFixedString)
        self._link.setCurrentIndex(unlink_index)
        
    def update_bounds(self):
        """
        update min/max bounds and check if range needs to be updated as well
        """
        bounds = [self._min, self._max]
        self._fitter.update_bounds(self._param_name, bounds, self._exp)

        # check if bounds are smaller than range, then update.
        curr_range = self._exp.model.param_guess_ranges[self._param_name]
        curr_bounds = self._exp.model.bounds[self._param_name]

        if curr_range[0] < curr_bounds[0] or curr_range[1] > curr_bounds[1]:
            self._fitter.update_range(self._param_name, bounds, self._exp)




from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

from .base import Experiments
from .. import slider_popup
from .. import sliders

class ConnectorsBox(Experiments):
    """
    hold global parameter/sliders
    """
    def __init__(self, name, connector, parent):
        """
        """
        self._connector = connector
        self._linked_list = []
        self._main_box = parent._main_box
        self._exp = None

        super().__init__(name, parent)

    def exp_widgets(self):
        """
        create slider
        """
        self._remove_name = "Unlink From All"
        
        # see if global variable is a connector or simple var
        param = self._connector.params

        for p, v in param.items():
            s = sliders.GlobalSliders(p, self)
            self._slider_list["Global"][self._name].append(s)

    def slider_popup(self):
        """
        hide and show slider window
        """
        self._slider_window = slider_popup.ConnectorPopUp(self)
        self._slider_window.show()

    def linked(self, loc_slider):
        """
        """
        self._linked_list.append(loc_slider)

    def unlinked(self, loc_slider):
        """
        remove item from linked list if local param unlinked from global param
        """
        self._linked_list.remove(loc_slider)

        # if nothing linked, delete the glob exp object
        if len(self._linked_list) == 0:
            self.remove()

    def remove(self):
        """
        """
        try:
            self._fitter.remove_global(self._name)
            self._slider_list["Global"].pop(self._name, None)
            self._global_tracker.pop(self._name, None)
            
            for s in self._linked_list:
                s.reset()
        except:
            pass

        self.close()



__author__ = "Hiranmayi Duvvuri"
__date__ = "2017-01-08"
__all__ = []

from .base import *
from .local_sliders import *
from .global_sliders import *
from .add_global_connector import AddGlobalConnectorWindow


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import string, random, inspect, ast

import pytc

class AddGlobalConnectorWindow(QDialog):
    """
    Construct a window that allows the user to build a GlobalConnector object.
    """

    def __init__(self, end_function):
        """
        :param end_function: a function to call when the "OK" button is called.  This
                             should take the connector and param_name to use.
        """
    

        super().__init__()

        self._end_function = end_function

        # Figure out the global connectors loaded
        possible_subclasses = pytc.global_connectors.GlobalConnector.__subclasses__()
        self._global_connectors = dict([(s.__name__,s) for s in possible_subclasses])

        self.layout()

    def layout(self):
        """
        Populate the window.
        """

        main_layout = QVBoxLayout(self)
        self._form_layout = QFormLayout()

        # Combobox widget holding possible connectors
        self._connector_select_widget = QComboBox(self)
        connector_names = list(self._global_connectors.keys())
        connector_names.sort()
        for n in connector_names:
            self._connector_select_widget.addItem(n)
        self._connector_select_widget.setCurrentIndex(0)

        # Connector selection call back
        self._connector_select_widget.activated[str].connect(self._update_connector)

        # Input box holding name
        self._connector_name_input = QLineEdit(self)

        random_name = "".join([random.choice(string.ascii_letters) for i in range(3)])
        self._connector_name_input.setText(random_name)

        # Connector name call back
        self._connector_name_input.textChanged[str].connect(self._update_connector_name)

        # Final OK button
        self._OK_button = QPushButton("OK", self)
        self._OK_button.clicked.connect(self._return_final_connector)

        # add to form
        self._form_layout.addRow(QLabel("Select Model:"), self._connector_select_widget)
        self._form_layout.addRow(QLabel("Name:"), self._connector_name_input)

        # Populate widgets
        self._arg_widgets = {}
        self._update_connector()

        # add to main layout
        main_layout.addLayout(self._form_layout)
        main_layout.addWidget(self._OK_button)

        self.setWindowTitle('Add new global connector')

    def _update_connector(self):
        """
        If the user selects a new connector, repopulate the window appropriately.
        """

        self.adjustSize()

        # remove args from form layout
        for l in self._arg_widgets.values():
            widget_labeled = self._form_layout.labelForField(l)
            if widget_labeled is not None:
                widget_labeled.deleteLater()
            l.deleteLater()

        # check if parameter label even there
        try:
            param_label = self._form_layout.labelForField(self._parameter_name_box)
            if param_label is not None:
                param_label.deleteLater()
            self._parameter_name_box.deleteLater()
        except:
            pass

        # Grab connector and name
        self._selected_connector_key = self._connector_select_widget.currentText()
        self._connector_name = self._connector_name_input.text()

        # Connector class   
        connector = self._global_connectors[self._selected_connector_key]

        # Create instance of connector class
        self._selected_connector = connector(name=self._connector_name)

        # Figure out the args that are specific to this connector
        new_fields = {} 
        connector_args = inspect.getargspec(connector.__init__)
        try:
            offset = len(connector_args.args) - len(connector_args.defaults)
            for i in range(len(connector_args.defaults)):
                new_fields[connector_args.args[i + offset]] = connector_args.defaults[i]

        # TypeError indicates no args...
        except TypeError:
            pass

        # Add the newly selected args back into the connector
        self._arg_widgets = {}
        for k, v in new_fields.items():

            label = QLabel(k,self)
            box = QLineEdit(self)
            box.setText("{}".format(v))

            self._arg_widgets[k] = box

            self._form_layout.addRow(label, box)

        # Create dropdown box for the parameter name to select
        label = QLabel("Select parameter",self)
        self._parameter_name_box = QComboBox(self)

        # Add the dropdown box to the list
        self._form_layout.addRow(label, self._parameter_name_box)

        # Update the list of parameters that can be selected
        self._update_param_box()

    def _update_connector_name(self):
        """
        If the user alters the name of the connector, update the parameters.
        """

        # Change inside of connector and update dropdown box with parameters to select
        self._selected_connector.name = self._connector_name_input.text()
        self._update_param_box()

    def _update_param_box(self):
        """
        Update the parameter box.
        """
        
        # Clear existing entries in the dropbox (it's the last widget)
        self._parameter_name_box.clear()

        # Update the names of the parameters
        self._param_names = list(self._selected_connector.local_methods.keys())
        self._param_names.sort()
        for k in self._param_names:
            self._parameter_name_box.addItem(k)
        
    def _return_final_connector(self):
        """
        Construct the final connector, run the end_function, and close.
        """

        # Grab connector and name
        self._selected_connector_key = self._connector_select_widget.currentText()
        self._connector_name = self._connector_name_input.text()
    
        # Connector class   
        connector = self._global_connectors[self._selected_connector_key]

        # Create kwargs to initialize the connector
        kwargs = {} 
        for k, widget in self._arg_widgets.items():
        
            # Parse the value.  
            value = widget.text()

            try:
                final_value = ast.literal_eval(value)
            except ValueError:
                if value.lower() in ["true","t"]:
                    final_value = True
                elif value.lower() in ["false","f"]:
                    final_value = False
                elif value.lower() == "none":
                    final_value = None
                else:
                    final_value = str(value)

            kwargs[k] = final_value

        # Create connector
        self._selected_connector = connector(name=self._connector_name,
                                             **kwargs)

        # Get currently selected parameter name
        var_name = self._parameter_name_box.currentText()

        # Pass data back
        self._end_function(self._selected_connector,self._param_names,var_name)

        self.close()
    



#!/usr/bin/env python3

import sys

if sys.version_info[0] < 3:
    sys.exit('Sorry, Python < 3.x is not supported')

# Try using setuptools first, if it's installed
from setuptools import setup, find_packages

# Need to add all dependencies to setup as we go!
setup(name='pytc-gui',
      packages=find_packages(exclude=['pytc']),
      version='0.1.1',
      description="PyQt5 GUI for pytc API",
      author='Hiranmayi Duvvuri',
      author_email='hiranmayid8@gmail.com',
      url='https://github.com/harmslab/pytc-gui',
      download_url='https://github.com/harmslab/pytc-gui/tarball/0.1.0',
      zip_safe=False,
      install_requires=["pytc-fitter","seaborn","pyqt5"],
      classifiers=['Programming Language :: Python'],
      scripts=['scripts/pytc-gui'])




from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
import seaborn
from io import StringIO

from .exp_frames import LocalBox, GlobalBox, ConnectorsBox
import pytc

class PlotBox(QWidget):
    """
    hold plot widget and update plot
    """

    def __init__(self, parent):
        """
        """
        super().__init__()

        self._fitter = parent._fitter

        self.layout()

    def layout(self):
        """
        """
        self._main_layout = QVBoxLayout(self)

    def update(self):
        """
        clear main layout and add new graph to layout
        """
        self.clear()
        self._figure, self._ax = self._fitter.plot()

        plot_figure = FigureCanvas(self._figure)
        self._main_layout.addWidget(plot_figure)

    def clear(self):
        """
        clear table
        """
        for i in range(self._main_layout.count()): 
            self._main_layout.itemAt(i).widget().deleteLater()

class ParamTable(QTableWidget):
    """
    take csv style param string and put into table widget
    """

    def __init__(self, fitter):
        super().__init__()

        self._fitter = fitter
        self._header = []
        self._col_name = []
        self._data = []

    def load_table(self):
        """
        load fit data into the table
        """
        for i, row in enumerate(self._data):
            for j, col in enumerate(row):
                item = QTableWidgetItem(col)
                self.setItem(i, j, item)

    def csv_to_table(self):
        """
        convert csv data file to lists to be read by qtablewidget
        """
        self._header = []
        self._col_name = []
        self._data = []

        file_data = self._fitter.fit_as_csv
        string_file = StringIO(file_data)

        # break up the file data
        for i in string_file:
            if i.startswith("#"):
                self._header.append(i.rstrip())
            elif i.startswith("type"):
                i = i.rstrip().split(',')
                self._col_name = i
            else:
                i = i.rstrip().split(',')
                self._data.append(i)

    def update(self):
        """
        update the table with updated fit parameters
        """
        self.csv_to_table()

        self.setRowCount(len(self._data))
        self.setColumnCount(len(self._data[0]))
        self.setHorizontalHeaderLabels(self._col_name)

        self.load_table()

    def clear(self):
        """
        """
        super().clear()
        self._header = []
        self._col_name = []
        self._data = []

class AllExp(QWidget):
    """
    experiment box widget
    """

    fit_signal = pyqtSignal()

    def __init__(self, parent):
        """
        """
        super().__init__()

        self._fitter = parent._fitter
        self._slider_list = {"Global" : {}, "Local" : {}}
        self._global_var = []
        self._global_tracker = {}
        self._global_connectors = {}
        self._connectors_seen = {}
        self._plot_frame = parent._plot_frame

        self.layout()

    def layout(self):
        """
        """
        self._main_layout = QVBoxLayout(self)

        # scroll box for experiments
        self._scroll = QScrollArea(self)
        self._exp_content = QWidget()
        self._exp_box = QVBoxLayout(self._exp_content)
        self._scroll.setWidget(self._exp_content)
        self._scroll.setWidgetResizable(True)
        self._exp_box.setAlignment(Qt.AlignTop)

        # paramater table
        self._param_box = ParamTable(self._fitter)

        # splitter for experiments and parameter widgets
        self._splitter = QSplitter(Qt.Vertical)
        self._splitter.addWidget(self._scroll)
        self._splitter.addWidget(self._param_box)
        self._splitter.setSizes([200, 200])

        self._main_layout.addWidget(self._splitter)

    def add_exp(self):
        """
        update fit and parameters, update experiments added to fitter
        """
        self._experiments = self._fitter.experiments

        if len(self._experiments) != 0:
            # create local holder if doesn't exist
            for e in self._experiments:
                if e in self._slider_list["Local"]:
                    continue

                self._slider_list["Local"][e] = []
                self._connectors_seen[e] = []

                file_name = e.dh_file
                exp_name = file_name.split("/")[-1]

                exp = LocalBox(e, exp_name, self)
                self._exp_box.addWidget(exp)
        else:
            print("no experiments loaded in fitter")
            self._param_box.clear()

    def perform_fit(self):
        """
        perform complete fit, update all
        """
        self.add_exp()

        # check for instances of LocalBox and set any attributes
        for loc_obj in self._exp_box.parentWidget().findChildren(LocalBox):
            loc_obj.set_attr()

        try:
            # after doing fit, emit signal to sliders and update parameter table
            self._fitter.fit()
            self.fit_signal.emit()

            # put fitting update here.
            self._param_box.update()
        except:
            fit_status = self._fitter.fit_status
            error_message = QMessageBox.warning(self, "warning! fit unsuccesful", "please enter experiment data", QMessageBox.Ok)

    def clear(self):
        """
        for clearing the application
        """
        try:
            # check for any global/connector vars, remove them first
            for i in range(self._exp_box.count()): 
                widget = self._exp_box.itemAt(i).widget()
                if isinstance(widget, LocalBox):
                    continue

                widget.remove()

            # finally, remove local objects
            for loc_obj in self._exp_box.parentWidget().findChildren(LocalBox):
                self._fitter.remove_experiment(loc_obj._exp)
                loc_obj.deleteLater()
        except:
            pass

        # reset all lists/dictionaries
        self._slider_list = {"Global" : {}, "Local" : {}}
        self._global_var = []
        self._connectors_seen = {}
        self._global_connectors = {}
        self._global_tracker = {}

        self._param_box.clear()



from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

import pytc
import inspect

from .. import sliders
from .base import SliderPopUp

class LocalPopUp(SliderPopUp):
    """
    pop-up window for slider widgets
    """

    def __init__(self, parent):
        """
        """
        self._global_var = parent._global_var
        self._connectors_seen = parent._connectors_seen

        super().__init__(parent)

    def populate(self):
        """
        """

        sliders = self._slider_list["Local"][self._exp]
        sliders.sort(key = lambda x: x.name)

        # add sliders to layout
        for s in sliders:
            self._main_layout.addWidget(s)